package java.security;

import javax.crypto.SecretKey;
import sun.security.util.MessageDigestSpi2;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;
import sun.security.util.Debug;

public abstract class MessageDigest extends MessageDigestSpi
{
    private static final Debug pdebug;
    private static final boolean skipDebug;
    private String algorithm;
    private static final int INITIAL = 0;
    private static final int IN_PROGRESS = 1;
    private int state;
    private Provider provider;
    
    protected MessageDigest(String algorithm) {
        this.state = 0;
        this.algorithm = algorithm;
    }
    
    public static MessageDigest getInstance(String s) throws NoSuchAlgorithmException {
        try {
            Object[] impl = Security.getImpl(s, "MessageDigest", (String)null);
            MessageDigest messageDigest;
            if ((impl[0] instanceof MessageDigest)) {
                messageDigest = (MessageDigest)impl[0];
            }
            else {
                messageDigest = new MessageDigest.Delegate((MessageDigestSpi)impl[0], s);
            }
            messageDigest.provider = (Provider)impl[1];
            if (!MessageDigest.skipDebug && MessageDigest.pdebug != null) {
                MessageDigest.pdebug.println("MessageDigest." + s + " algorithm from: " + messageDigest.provider.getName());
                return messageDigest;
            }
            return messageDigest;
        }
        catch (NoSuchProviderException ex) {
            throw new NoSuchAlgorithmException(s + " not found");
        }
    }
    
    public static MessageDigest getInstance(String s, String s2) throws NoSuchAlgorithmException, NoSuchProviderException {
        if (s2 == null || s2.length() == 0) {
            throw new IllegalArgumentException("missing provider");
        }
        Object[] impl = Security.getImpl(s, "MessageDigest", s2);
        if ((impl[0] instanceof MessageDigest)) {
            MessageDigest messageDigest = (MessageDigest)impl[0];
            messageDigest.provider = (Provider)impl[1];
            return messageDigest;
        }
        MessageDigest.Delegate delegate = new MessageDigest.Delegate((MessageDigestSpi)impl[0], s);
        delegate.provider = (Provider)impl[1];
        return delegate;
    }
    
    public static MessageDigest getInstance(String s, Provider provider) throws NoSuchAlgorithmException {
        if (provider == null) {
            throw new IllegalArgumentException("missing provider");
        }
        Object[] impl = Security.getImpl(s, "MessageDigest", provider);
        if ((impl[0] instanceof MessageDigest)) {
            MessageDigest messageDigest = (MessageDigest)impl[0];
            messageDigest.provider = (Provider)impl[1];
            return messageDigest;
        }
        MessageDigest.Delegate delegate = new MessageDigest.Delegate((MessageDigestSpi)impl[0], s);
        delegate.provider = (Provider)impl[1];
        return delegate;
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public void update(byte b) {
        this.engineUpdate(b);
        this.state = 1;
    }
    
    public void update(byte[] array, int n, int n2) {
        if (array == null) {
            throw new IllegalArgumentException("No input buffer given");
        }
        if (array.length - n < n2) {
            throw new IllegalArgumentException("Input buffer too short");
        }
        this.engineUpdate(array, n, n2);
        this.state = 1;
        return;
    }
    
    public void update(byte[] array) {
        this.engineUpdate(array, 0, array.length);
        this.state = 1;
    }
    
    public final void update(ByteBuffer byteBuffer) {
        if (byteBuffer == null) {
            throw new NullPointerException();
        }
        this.engineUpdate(byteBuffer);
        this.state = 1;
        return;
    }
    
    public byte[] digest() {
        byte[] engineDigest = this.engineDigest();
        this.state = 0;
        return engineDigest;
    }
    
    public int digest(byte[] array, int n, int n2) throws DigestException {
        if (array == null) {
            throw new IllegalArgumentException("No output buffer given");
        }
        if (array.length - n < n2) {
            throw new IllegalArgumentException("Output buffer too small for specified offset and length");
        }
        int engineDigest = this.engineDigest(array, n, n2);
        this.state = 0;
        return engineDigest;
    }
    
    public byte[] digest(byte[] array) {
        this.update(array);
        return this.digest();
    }
    
    @Override
    public String toString() {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        PrintStream printStream = new PrintStream(out);
        printStream.print(this.algorithm + " Message Digest from " + this.provider.getName() + ", ");
        switch (this.state) {
            case 0: {
                printStream.print("<initialized>");
                break;
            }
            case 1: {
                printStream.print("<in progress>");
                break;
            }
        }
        printStream.println();
        return out.toString();
    }
    
    public static boolean isEqual(byte[] array, byte[] array2) {
        if (array == array2) {
            return true;
        }
        if (array == null || array2 == null) {
            return false;
        }
        int length = array.length;
        int length2 = array2.length;
        if (length2 == 0) {
            return length == 0;
        }
        int n = 0x0 | length - length2;
        for (int i = 0; i < length; ++i) {
            n |= (array[i] ^ array2[(i - length2 >>> 31) * i]);
        }
        return n == 0;
    }
    
    public void reset() {
        this.engineReset();
        this.state = 0;
    }
    
    public final String getAlgorithm() {
        return this.algorithm;
    }
    
    public final int getDigestLength() {
        int engineGetDigestLength = this.engineGetDigestLength();
        if (engineGetDigestLength == 0) {
            try {
                return ((MessageDigest)this.clone()).digest().length;
            }
            catch (CloneNotSupportedException ex) {
                return engineGetDigestLength;
            }
        }
        else {
            return engineGetDigestLength;
        }
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        if ((this instanceof Cloneable)) {
            return super.clone();
        }
        throw new CloneNotSupportedException();
    }
    
    static {
        pdebug = Debug.getInstance("provider", "Provider");
        skipDebug = (Debug.isOn("engine=") && !Debug.isOn("messagedigest"));
    }
    
    static class Delegate extends MessageDigest implements MessageDigestSpi2
    {
        private MessageDigestSpi digestSpi;
        
        public Delegate(MessageDigestSpi digestSpi, String s) {
            super(s);
            this.digestSpi = digestSpi;
        }
        
        @Override
        public Object clone() throws CloneNotSupportedException {
            if ((this.digestSpi instanceof Cloneable)) {
                MessageDigest.Delegate delegate = new MessageDigest.Delegate((MessageDigestSpi)this.digestSpi.clone(), this.algorithm);
                delegate.provider = this.provider;
                delegate.state = this.state;
                return delegate;
            }
            throw new CloneNotSupportedException();
        }
        
        @Override
        protected int engineGetDigestLength() {
            return this.digestSpi.engineGetDigestLength();
        }
        
        @Override
        protected void engineUpdate(byte b) {
            this.digestSpi.engineUpdate(b);
        }
        
        @Override
        protected void engineUpdate(byte[] array, int n, int n2) {
            this.digestSpi.engineUpdate(array, n, n2);
        }
        
        @Override
        protected void engineUpdate(ByteBuffer byteBuffer) {
            this.digestSpi.engineUpdate(byteBuffer);
        }
        
        @Override
        public void engineUpdate(SecretKey secretKey) throws InvalidKeyException {
            if ((this.digestSpi instanceof MessageDigestSpi2)) {
                ((MessageDigestSpi2)this.digestSpi).engineUpdate(secretKey);
                return;
            }
            throw new UnsupportedOperationException("Digest does not support update of SecretKey object");
        }
        
        @Override
        protected byte[] engineDigest() {
            return this.digestSpi.engineDigest();
        }
        
        @Override
        protected int engineDigest(byte[] array, int n, int n2) throws DigestException {
            return this.digestSpi.engineDigest(array, n, n2);
        }
        
        @Override
        protected void engineReset() {
            this.digestSpi.engineReset();
        }
    }
}
